/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.test;

import it.tidalwave.semantic.other.Repository;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.util.Id;
import it.tidalwave.semantic.other.Type;
import java.io.PrintWriter;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public abstract class EntityTestSupport<C extends Repository, T extends Type, E extends Entity> extends TestSupport<C>
  {
    private Id entityId1;
    private Id entityId2;
    private Type entityType1;
    private Type entityType2;
    protected Entity entity1;
    protected Entity entity2;
    private Type propertyType1;
    private Type propertyType2;

    @Before
    public final void setUp()
      throws Exception
      {
        entityId1 = new Id(NS + "eid1");
        entityId2 = new Id(NS + "eid2");
        entityType1 = repository.findOrCreateType(new Id(NS + "Class1"), "Class 1");
        entityType2 = repository.findOrCreateType(new Id(NS + "Class2"), "Class 2");
        entity1 = entityType1.findOrCreateEntity(entityId1, "Entity 1");
        entity2 = entityType2.findOrCreateEntity(entityId2, "Entity 2");
        propertyType1 = repository.findOrCreateType(new Id(NS + "PropertyClass1"), "Property Class 1");
        propertyType2 = repository.findOrCreateType(new Id(NS + "PropertyClass2"), "Property Class 2");
      }

    @Test
    public final void testStringValue()
      {
        assertEquals(NS + "eid1", entity1.stringValue());
        assertEquals(NS + "eid2", entity2.stringValue());
      }

    @Test
    public final void testGetType()
      {
        assertSame(entityType1, entity1.getType());
        assertSame(entityType2, entity2.getType());
      }

    @Test
    public final void testGetId()
      {
//        assertSame(entityId1, entity1.getId());
//        assertSame(entityId2, entity2.getId());
        assertEquals(entityId1, entity1.getId());
        assertEquals(entityId2, entity2.getId());
      }

    @Test
    public final void testSetGetProperty()
      throws Exception
      {
        assertNull(entity1.getProperty(propertyType1));
        assertNull(entity1.getProperty(propertyType2));

        entity1.setProperty(propertyType1, "value1");
        entity1.setProperty(propertyType2, "value2");

        assertEquals("value1", entity1.getProperty(propertyType1));
        assertEquals("value2", entity1.getProperty(propertyType2));
        dump("testSetGetProperty()", repository, new PrintWriter(System.err, true));
      }

    @Test
    public final void testRemoveProperty()
      {
        // TBD
      }

    @Test
    public final void testGetProperties()
      {
        entity1.setProperty(propertyType1, "value1");
        entity1.setProperty(propertyType2, "value2");
        // TBD
      }

    @Test
    public final void testGetRelations()
      {
        assertTrue(entity1.findRelations().isEmpty());
        // TBD
      }

    @Test
    public final void testGetEquivalentIds()
      {
        // TBD
      }

    @Test
    public final void testAddRelation()
      {
      }

    @Test
    public final void testGetDisplayName_Locale()
      {
        // TBD
      }

    @Test
    public final void testGetDisplayName()
      {
        assertEquals("Entity 1", entity1.getDisplayName());
        assertEquals("Entity 2", entity2.getDisplayName());
      }

    @Test
    public final void testGetLocales()
      {
        // TBD
      }

    @Test
    public final void testCompareTo()
      {
        assertEquals(-1, entity1.compareTo(entity2));
        assertEquals(+1, entity2.compareTo(entity1));
      }

    @Test
    public final void testEquals()
      {
        assertFalse(entity1.equals(entity2));
        assertFalse(entity2.equals(entity1));
      }

    @Test
    public final void testHashCode()
      {
        // TBD compare hash of two equivalent Entities
      }


    @Test
    public final void testToString()
      {
        assertEquals(String.format("OpenRDFEntity[%seid1 (Entity 1): OpenRDFType[%sClass1 (Class 1)]]", NS, NS), entity1.toString());
        assertEquals(String.format("OpenRDFEntity[%seid2 (Entity 2): OpenRDFType[%sClass2 (Class 2)]]", NS, NS), entity2.toString());
      }
  }
