/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.semantic.test;

import java.util.Iterator;
import java.util.SortedSet;
import java.io.PrintWriter;
import it.tidalwave.util.Id;
import it.tidalwave.semantic.other.Repository;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.other.Type;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public abstract class TypeTestSupport<C extends Repository, T extends Type> extends TestSupport<C>
  {
    private Id id1;
    private Id id1b;
    private Id id2;
    protected T type1;
    protected T type1b;
    protected T type2;

    @Before
    public final void setUp()
      {
        id1 = new Id(NS + "Class1");
        id1b = new Id(NS + "Class1");
        id2 = new Id(NS + "Class2");
        type1 = (T)repository.findOrCreateType(id1, "Class 1");
        type1b = (T)repository.findOrCreateType(id1b, "Class 1");
        type2 = (T)repository.findOrCreateType(id2, "Class 2");
      }

    @Test
    public final void testGetId()
      {
//        assertSame(id1, type1.getId());
//        assertSame(id1b, type1b.getId());
//        assertSame(id2, type2.getId());
        assertEquals(id1, type1.getId());
        assertEquals(id1b, type1b.getId()); // FIXME
        assertEquals(id2, type2.getId());
      }

    @Test
    public final void testStringValue()
      {
        assertEquals(NS + "Class1", type1.stringValue());
        assertEquals(NS + "Class1", type1b.stringValue());
        assertEquals(NS + "Class2", type2.stringValue());
      }

    @Test
    public void testGetDisplayName_Locale()
      {
        // TBD
      }

    @Test
    public final void testGetDisplayName()
      {
        assertEquals("Class 1", type1.getDisplayName());
        assertEquals("Class 1", type1b.getDisplayName());
        assertEquals("Class 2", type2.getDisplayName());
      }

    @Test
    public final void testGetLocales()
      {
        // TBD
      }

    @Test
    public final void testFindOrCreateEntity()
      throws Exception
      {
        final Id eid1 = new Id(NS + "EntityType1");
        final Id eid1b = new Id(NS + "EntityType1");
        final Id eid2 = new Id(NS + "EntityType2");

        final Entity entityType1 = type1.findOrCreateEntity(eid1, "Entity Type 1");
        assertNotNull(entityType1);
        final Entity entityType1b = type1.findOrCreateEntity(eid1b, "Entity Type 1");
        assertNotNull(entityType1);
        final Entity entityType2 = type1.findOrCreateEntity(eid2, "Entity Type 2");
        assertNotNull(entityType1);

        assertNotSame(entityType1, entityType2);
        assertSame(entityType1, entityType1b);

        dump("testFindOrCreateEntity()", repository, new PrintWriter(System.err, true));
      }

    @Test
    public void testGetAllEntities()
      throws Exception
      {
        final Entity entity1 = type1.findOrCreateEntity(new Id(NS + "Entity1"), "Entity 1");
        final Entity entity2 = type1.findOrCreateEntity(new Id(NS + "Entity2"), "Entity 2");
        final Entity entity3 = type2.findOrCreateEntity(new Id(NS + "Entity3"), "Entity 3");
        final Entity entity4 = type2.findOrCreateEntity(new Id(NS + "Entity4"), "Entity 4");

        final SortedSet<Entity> allEntitiesType1 = type1.getAllEntities();
        assertEquals(2, allEntitiesType1.size());
        final Iterator<Entity> i1 = allEntitiesType1.iterator();
        assertSame(entity1, i1.next());
        assertSame(entity2, i1.next());

        final SortedSet<Entity> allEntitiesType2 = type2.getAllEntities();
        assertEquals(2, allEntitiesType2.size());
        final Iterator<Entity> i2 = allEntitiesType2.iterator();
        assertSame(entity3, i2.next());
        assertSame(entity4, i2.next());

        dump("testGetAllEntities()", repository, new PrintWriter(System.err, true));
      }

    @Test
    public final void testCompareTo()
      {
        assertEquals(0, type1.compareTo(type1b));
        assertEquals(0, type1b.compareTo(type1));
        assertEquals(-1, type1.compareTo(type2));
        assertEquals(+1, type2.compareTo(type1));
      }

    @Test
    public final void testEquals()
      {
        assertTrue(type1.equals(type1b));
        assertTrue(type1b.equals(type1));
        assertFalse(type1.equals(type2));
        assertFalse(type2.equals(type1));
      }

    @Test
    public final void testHashCode()
      {
        assertEquals(type1.hashCode(), type1b.hashCode());
      }


    @Test
    public final void testToString()
      {
        assertEquals(String.format("OpenRDFType[%sClass1 (Class 1)]", NS), type1.toString());
        assertEquals(String.format("OpenRDFType[%sClass1 (Class 1)]", NS), type1b.toString());
        assertEquals(String.format("OpenRDFType[%sClass2 (Class 2)]", NS), type2.toString());
      }
  }